@@ -387,9 +387,9 @@ def rdump(path: str, data: Dict) -> None:
 
 def rload(fname: str) -> dict:
     """Parse data and parameter variable values from an R dump format file.
-       This parser only supports the subset of R dump data as described
-       in the "Dump Data Format" section of the CmdStan manual, i.e.,
-       scalar, vector, matrix, and array data types.
+    This parser only supports the subset of R dump data as described
+    in the "Dump Data Format" section of the CmdStan manual, i.e.,
+    scalar, vector, matrix, and array data types.
     """
     data_dict = {}
     with open(fname, 'r') as fd:
@@ -420,8 +420,8 @@ def rload(fname: str) -> dict:
 
 def parse_rdump_value(rhs: str) -> Union[int, float, np.array]:
     """Process right hand side of Rdump variable assignment statement.
-       Value is either scalar, vector, or multi-dim structure.
-       Use regex to capture structure values, dimensions.
+    Value is either scalar, vector, or multi-dim structure.
+    Use regex to capture structure values, dimensions.
     """
     pat = re.compile(
         r'structure\(\s*c\((?P<vals>[^)]*)\)'
@@ -444,8 +444,8 @@ def parse_rdump_value(rhs: str) -> Union[int, float, np.array]:
             val = float(rhs)
         else:
             val = int(rhs)
-    except TypeError:
-        raise ValueError('bad value in Rdump file: {}'.format(rhs))
+    except TypeError as exc:
+        raise ValueError('bad value in Rdump file: {}'.format(rhs)) from exc
     return val
 
 
@@ -553,27 +553,26 @@ def scan_variational_csv(path: str) -> Dict:
         lineno = scan_column_names(fd, dict, lineno)
         line = fd.readline().lstrip(' #\t').rstrip()
         lineno += 1
-        if not line.startswith('Stepsize adaptation complete.'):
-            raise ValueError(
-                'line {}: expecting adaptation msg, found:\n\t "{}"'.format(
-                    lineno, line
+        if line.startswith('Stepsize adaptation complete.'):
+            line = fd.readline().lstrip(' #\t\n')
+            lineno += 1
+            if not line.startswith('eta'):
+                raise ValueError(
+                    'line {}: expecting eta, found:\n\t "{}"'.format(
+                        lineno, line
+                    )
                 )
-            )
-        line = fd.readline().lstrip(' #\t\n')
-        lineno += 1
-        if not line.startswith('eta = 1'):
-            raise ValueError(
-                'line {}: expecting eta = 1, found:\n\t "{}"'.format(
-                    lineno, line
-                )
-            )
-        line = fd.readline().lstrip(' #\t\n')
-        lineno += 1
+            line = fd.readline().lstrip(' #\t\n')
+            lineno += 1
         xs = line.split(',')
         variational_mean = [float(x) for x in xs]
         dict['variational_mean'] = variational_mean
         dict['variational_sample'] = pd.read_csv(
-            path, comment='#', skiprows=lineno, header=None
+            path,
+            comment='#',
+            skiprows=lineno,
+            header=None,
+            float_precision='high',
         )
     return dict
 
@@ -691,10 +690,10 @@ def scan_metric(fd: TextIO, config_dict: Dict, lineno: int) -> int:
         )
     try:
         float(stepsize.strip())
-    except ValueError:
+    except ValueError as exc:
         raise ValueError(
             'line {}: invalid stepsize: {}'.format(lineno, stepsize)
-        )
+        ) from exc
     line = fd.readline().strip()
     lineno += 1
     if not (
