@@ -117,7 +117,7 @@ class GenerateQuantitiesTest(unittest.TestCase):
         ]
         self.assertEqual(bern_gqs.column_names, tuple(column_names))
         self.assertEqual(
-            bern_fit.get_drawset().shape, bern_gqs.mcmc_sample.shape
+            bern_fit.draws_as_dataframe().shape, bern_gqs.mcmc_sample.shape
         )
         self.assertEqual(
             bern_gqs.sample_plus_quantities.shape[1],
