@@ -31,6 +31,7 @@ model {
 
 BERN_STAN = os.path.join(DATAFILES_PATH, 'bernoulli.stan')
 BERN_EXE = os.path.join(DATAFILES_PATH, 'bernoulli' + EXTENSION)
+BERN_BASENAME = 'bernoulli'
 
 
 class CmdStanModelTest(unittest.TestCase):
@@ -50,22 +51,24 @@ class CmdStanModelTest(unittest.TestCase):
         self.assertTrue(True)
 
     def test_model_good(self):
-        # compile on instantiation
+        # compile on instantiation, override model name
         model = CmdStanModel(model_name='bern', stan_file=BERN_STAN)
         self.assertEqual(BERN_STAN, model.stan_file)
         self.assertTrue(model.exe_file.endswith(BERN_EXE.replace('\\', '/')))
         self.assertEqual('bern', model.name)
 
+        # default model name
+        model = CmdStanModel(stan_file=BERN_STAN)
+        self.assertEqual(BERN_BASENAME, model.name)
+
         # instantiate with existing exe
         model = CmdStanModel(stan_file=BERN_STAN, exe_file=BERN_EXE)
         self.assertEqual(BERN_STAN, model.stan_file)
         self.assertTrue(model.exe_file.endswith(BERN_EXE))
-        self.assertEqual('bernoulli', model.name)
 
         # instantiate with existing exe only - no model
         model2 = CmdStanModel(exe_file=BERN_EXE)
         self.assertEqual(BERN_EXE, model2.exe_file)
-        self.assertEqual('bernoulli', model2.name)
         with self.assertRaises(RuntimeError):
             model2.code()
         with self.assertRaises(RuntimeError):
@@ -82,6 +85,10 @@ class CmdStanModelTest(unittest.TestCase):
             CmdStanModel(stan_file=None, exe_file=None)
         with self.assertRaises(ValueError):
             CmdStanModel(model_name='bad')
+        with self.assertRaises(ValueError):
+            CmdStanModel(model_name='', stan_file=BERN_STAN)
+        with self.assertRaises(ValueError):
+            CmdStanModel(model_name='   ', stan_file=BERN_STAN)
 
     def test_stanc_options(self):
         opts = {
